//
//  IHAlphaDataView.m
//
//  Created by iOS Hero on 12/23/11.
//  Copyright (c) 2011 iOS Hero. All rights reserved.
//

#import "IHAlphaDataView.h"

#define kAlphaThreshold 0.01f
#define kShowTracks NO

@interface IHAlphaDataView ()
{
    byte * alphaData;
}

- (void)cacheAlphaData;
- (void)showTrack:(CGPoint)point;

@end

@implementation IHAlphaDataView

- (void)awakeFromNib
{
    [self cacheAlphaData];    
}

- (BOOL)pointInsideImage:(CGPoint)point
{    
    CGFloat alpha = [self alphaAtPoint:point];
    
    BOOL transparent = (alpha < kAlphaThreshold);    
    
    NSLog(transparent ? @"out" : @"in");
    
    return ! transparent;
}

- (BOOL)pointInsideImageUsingCache:(CGPoint)point
{
    CGFloat alpha = [self alphaAtPointUsingCache:point];
    
    BOOL transparent = (alpha < kAlphaThreshold);    
    
    NSLog(transparent ? @"out" : @"in");
    
    return ! transparent;
}

- (float)alphaAtPoint:(CGPoint)point
{
    if(kShowTracks)
    {
        [self showTrack:point];
    }
    
    byte data = 0; //make sure this is initialized to 0
    
    CGContextRef context = CGBitmapContextCreate(&data, 1, 1, 8, sizeof(byte), NULL, kCGImageAlphaOnly);
    
    UIGraphicsPushContext(context);
    [imageView.image drawAtPoint:CGPointMake(-point.x, -point.y)];

    UIGraphicsPopContext();
    CGContextRelease(context);

    return data / 255.0f;
}

- (float)alphaAtPointUsingCache:(CGPoint)point
{
    if(! alphaData)
    {
        NSLog(@"alpha data cache has not been initialized");
        return 0.0f;
    }
    
    if(kShowTracks)
    {
        [self showTrack:point];
    }
    
    CGImageRef imageRef = imageView.image.CGImage;
    int width = CGImageGetWidth(imageRef);
    int height = CGImageGetHeight(imageRef);
    
    //take retina, non-retina scale into account
    float scale = [[UIScreen mainScreen] scale];
    int x = point.x * scale;
    int y = point.y * scale;
    
    if(x >= width || y >= height) return NO;
    
    int byteIndex = y * width + x;
    
    byte data = alphaData[byteIndex];
    
    return data / 255.0f;
}

- (void)cacheAlphaData
{
    CGImageRef imageRef = imageView.image.CGImage;
    
    int width = CGImageGetWidth(imageRef);
    int height = CGImageGetHeight(imageRef);
    
    int bitsPerComponent = 8;
    int byteCount = width * height;
    alphaData = malloc(byteCount);
    int bytesPerRow = width;
    CGBitmapInfo bitmapInfo = kCGImageAlphaOnly;
    
    CGContextRef context = CGBitmapContextCreate(alphaData, width, height, bitsPerComponent, bytesPerRow, NULL, bitmapInfo);
    CGContextSetBlendMode(context, kCGBlendModeCopy);
    CGContextDrawImage(context, CGRectMake(0, 0, width, height), imageRef);
    CGContextRelease(context);
}

- (void)showTrack:(CGPoint)point
{
    UIView * track = [[UIView alloc] initWithFrame:CGRectMake(point.x, point.y, 1, 1)];
    track.backgroundColor = [UIColor colorWithWhite:0 alpha:0.5];
    [self addSubview:track];
    [track performSelector:@selector(removeFromSuperview) withObject:nil afterDelay:2];
}

- (void)dealloc
{
    free(alphaData);    
}

@end
